@@ -223,7 +223,7 @@ bool Box_Data::confirm_discard_unstored_data() const
   if(get_unstored_data())
   {
     //Ask user to confirm loss of data:
-    Gtk::MessageDialog dialog(gettext("<b>No primary key value</b>"), true, Gtk::MESSAGE_QUESTION, (Gtk::ButtonsType)(Gtk::BUTTONS_OK | Gtk::BUTTONS_CANCEL) );
+    Gtk::MessageDialog dialog(gettext("<b>No primary key value</b>"), true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_OK_CANCEL );
     dialog.set_secondary_text(gettext("This data can not be stored in the database because you have not provided a primary key.\nDo you really want to discard this data?"));
     //TODO: dialog.set_transient_for(*this);
     int iButton = dialog.run();
@@ -403,15 +403,15 @@ Box_Data::type_vecLayoutFields Box_Data::get_table_fields_to_show(const Glib::us
   {
     g_warning("Box_Data::get_table_fields_to_show_add_group(): Returning empty list.");
   }
-  
+
   return result;
 }
 
-guint Box_Data::generate_next_auto_increment(const Glib::ustring& table_name, const Glib::ustring field_name)
+Gnome::Gda::Value Box_Data::generate_next_auto_increment(const Glib::ustring& table_name, const Glib::ustring field_name)
 {
   //This is a workaround for postgres problems. Ideally, we need to use the postgres serial type and find out how to get the generated value after we add a row.
 
-  guint result = 0;
+  double result = 0;
   Glib::RefPtr<Gnome::Gda::DataModel> data_model = Query_execute("SELECT max(" + field_name + ") FROM " + table_name);
   if(!data_model && data_model->get_n_rows())
     handle_error();
@@ -420,12 +420,18 @@ guint Box_Data::generate_next_auto_increment(const Glib::ustring& table_name, co
     //The result should be 1 row with 1 column
     Gnome::Gda::Value value = data_model->get_value_at(0, 0);
 
-    //It probably has a specific numeric type, but I am being lazy. murrayc
+    //It's a Gnome::Gda::Value::TYPE_NUMERIC, but the GdaNumeric struct is not easy to handle, so let's hack around it:
+    //if(value.is_number())
+    //  result = value.get_integer();
+    //else
     result = util_decimal_from_string(value.to_string());
-    ++result;
+
+    ++result; 
   }
 
-  return result;
+  //Get a string representation of the number, so we can put it back in a NUMERIC Gda::Value:
+
+  return GlomConversions::parse_value(result);
 }
 
 /** Get the fields that are in related tables, via a relationship using @a field_name changes.
@@ -770,3 +776,126 @@ Glib::ustring Box_Data::get_layout_item_table_name(const LayoutItem_Field& layou
 
   return Glib::ustring();
 }
+
+bool Box_Data::get_related_record_exists(const Relationship& relationship, const Field& key_field, const Gnome::Gda::Value& key_value)
+{
+  bool result = false;
+
+  //TODO_Performance: It's very possible that this is slow.
+  //We don't care how many records there are, only whether there are more than zero.
+  const Glib::ustring to_field = relationship.get_to_table() = relationship.get_to_field();
+
+  //TODO_Performance: Is this the best way to just find out whether there is one record that meets this criteria?
+  const Glib::ustring query = "SELECT " + to_field + " FROM " + relationship.get_to_table() + " WHERE " + to_field + " = " + key_field.sql(key_value);
+  Glib::RefPtr<Gnome::Gda::DataModel> records = Query_execute(query);
+  if(!records)
+    handle_error();
+  else
+  {
+    //Field contents:
+    if(records->get_n_rows())
+      result = true;
+  }
+
+  return result;
+}
+
+bool Box_Data::add_related_record_for_field(const LayoutItem_Field& layout_item_parent, const Relationship& relationship, const Field& primary_key_field, const Gnome::Gda::Value& primary_key_value_provided)
+{
+  Gnome::Gda::Value primary_key_value = primary_key_value_provided;
+
+  const bool related_record_exists = get_related_record_exists(relationship, primary_key_field, primary_key_value);
+  if(related_record_exists)
+  {
+    //No problem, the SQL command below will update this value in the related table.
+  }
+  else
+  {
+    //To store the entered data in the related field, we would first have to create a related record.
+    if(!relationship.get_auto_create())
+    {
+      //Warn the user:
+      //TODO: Make the field insensitive until it can receive data, so people never see this dialog.
+      Gtk::MessageDialog dialog(gettext("<b>Related Record Does Not Exist</b>"), true);
+      dialog.set_secondary_text(gettext("Data may not be entered into this related field, because the related record does not yet exist, and the relationship does not allow automatic creation of new related records."));
+      dialog.set_transient_for(*get_app_window());
+      dialog.run();
+
+      //Clear the field again, discarding the entered data.
+      set_entered_field_data(layout_item_parent, Gnome::Gda::Value());
+
+      return false;
+    }
+    else
+    {
+      const bool key_is_auto_increment = primary_key_field.get_field_info().get_auto_increment();
+
+      //If we would have to set an otherwise auto-increment key to add the record.
+      if( key_is_auto_increment && !GlomConversions::value_is_empty(primary_key_value) )
+      {
+        //Warn the user:
+        //TODO: Make the field insensitive until it can receive data, so people never see this dialog.
+        Gtk::MessageDialog dialog(gettext("<b>Related Record Can Not Be Created</b>"), true);
+        //TODO: This is a very complex error message:
+        dialog.set_secondary_text(gettext("Data may not be entered into this related field, because the related record does not yet exist, and the key in the related record is auto-generated and therefore can not be created with the key value in this record."));
+        dialog.set_transient_for(*get_app_window());
+        dialog.run();
+
+        //Clear the field again, discarding the entered data.
+        set_entered_field_data(layout_item_parent, Gnome::Gda::Value());
+
+        return false;
+      }
+      else
+      {
+        //TODO: Calculate values, and do lookups?
+
+        //Create the related record:
+        if(key_is_auto_increment)
+        {
+          primary_key_value = generate_next_auto_increment(relationship.get_to_table(), primary_key_field.get_name());
+
+          //Generate the new key value;
+        }
+
+        const Glib::ustring strQuery = "INSERT INTO " + relationship.get_to_table() + " (" + primary_key_field.get_name() + ") VALUES (" + primary_key_field.sql(primary_key_value) + ")";
+        bool test = Query_execute(strQuery);
+        if(test)
+        {
+          if(key_is_auto_increment)
+          {
+            //Set the key in the parent table
+            LayoutItem_Field item_from_key;
+            item_from_key.set_name(relationship.get_from_field());
+
+            //Show the new from key in the parent table's layout:
+            set_entered_field_data(item_from_key, primary_key_value);
+
+            //Set it in the database too:
+            Field field_from_key;
+            bool test = get_fields_for_table_one_field(relationship.get_from_table(), relationship.get_from_field(), field_from_key); //TODO_Performance.
+            if(test)
+            {
+              Field parent_primary_key_field;
+              bool test = get_field_primary_key(parent_primary_key_field);
+              if(!test)
+                return false;
+              else
+              {
+                const Gnome::Gda::Value parent_primary_key_value = get_primary_key_value_selected();
+
+                const Glib::ustring strQuery = "UPDATE  " + relationship.get_from_table() + " SET " + relationship.get_from_field() + " = " + primary_key_field.sql(primary_key_value) +
+                  " WHERE " + relationship.get_from_table() + "." + parent_primary_key_field.get_name() + " = " + parent_primary_key_field.sql(parent_primary_key_value);
+                bool test = Query_execute(strQuery);
+                return test;
+              }
+            }
+          }
+        }
+      }
+    }
+
+  }
+
+  return true;
+}
